import math
import matplotlib.pyplot as plt
import numpy as np
from .GeneralDistribution import Distribution

class Gaussian(Distribution):
	""" 
	Gaussian distribution class for calculating and visualizing a Gaussian distribution.
	
	Attributes:
		mean (float) representing the mean value of the distribution
		stdev (float) representing the standard deviation of the distribution
		data_list (list of floats) a list of floats extracted from the data file
			
	"""
	def __init__(self, mu=0, sigma=1):
		
		Distribution.__init__(self, mu, sigma)
		self.data = np.random.normal(self.mean, self.stdev, 500)

	
	def calculate_mean(self):
	
		"""
		Function to calculate the mean of the data set.
		
		Args: 
			None
		
		Returns: 
			float: mean of the data set
	
		"""
							
		self.mean = 1.0 * sum(self.data)/len(self.data)
		
		return self.mean
		

	def calculate_stdev(self, sample=True):

		""" 
		Function to calculate the standard deviation of the data set.
		
		Args: 
			sample (bool): whether the data represents a sample or population
		
		Returns: 
			float: standard deviation of the data set
	
		"""

		size = len(self.data) - int(sample == True)
		adjusted_data = [(num - self.mean) ** 2 for num in self.data]
		self.stdev = math.sqrt(sum(adjusted_data) / size)

		return self.stdev
		

	def read_data_file(self, file_name, sample=True):
	
		"""
		Function to read in data from a txt file. The txt file should have
		one number (float) per line. The numbers are stored in the data attribute. 
		After reading in the file, the mean and standard deviation are calculated
				
		Args:
			file_name (string): name of a file to read from
			sample (bool): whether the data represents a sample or population
	
		Returns:
			None
		
		"""
			
		Distribution.read_data_file(self, file_name)
		self.mean = self.calculate_mean()
		self.stdev = self.calculate_stdev(sample)
		
		
	def plot_histogram(self):
	
		"""
		Function to output a histogram of the instance variable data using 
		matplotlib pyplot library.
		
		Args:
			None
			
		Returns:
			None
		"""
		plt.hist(self.data)
		plt.title('Histogram of Data')
		plt.xlabel('Data')
		plt.ylabel('Count')
		plt.show()
		
		
	def pdf(self, x):
	
		"""
		Probability density function calculator for the gaussian distribution.
		
		Args:
			x (float): point for calculating the probability density function
			
		
		Returns:
			float: probability density function output
		"""
		
		return (1.0/(self.stdev * math.sqrt(2 * math.pi))) * math.exp(-0.5*((x - self.mean) / self.stdev) ** 2)
		
	def cdf(self, x):
	
		"""
		Cumulative density function calculator for the gaussian distribution.
		
		Args:
			x (float): point for calculating the cumulative density function
			
		
		Returns:
			float: cumulative density function output
		"""
		
		return .5 * (1 + math.erf((x - self.mean)/(self.stdev * math.sqrt(2))))

	def plot_histogram_pdf_cdf(self, n_spaces = 50):

		"""
		Function to plot the normalized histogram of the data and a plot of the 
		probability density function along the same range
		
		Args:
			n_spaces (int): number of data points 
		
		Returns:
            list: x values for the both plot
            list: y1 values for the pmf plot
            list: y2 values for cmf plot
			
		"""

		min_range = min(self.data)
		max_range = max(self.data)
		
		interval = 1.0 * (max_range - min_range) / n_spaces

		x = []
		y1 = []
		y2 = []
		
		for i in range(n_spaces):
			tmp = min_range + interval*i
			x.append(tmp)
			y1.append(self.pdf(tmp))
			y2.append(self.cdf(tmp))

		fig, axes = plt.subplots(3,sharex=True)
		fig.subplots_adjust(hspace=.5)
		
		axes[0].hist(self.data, n_spaces, density=True)
		axes[0].set_title('Normed Histogram of Data')
		axes[0].set_ylabel('Density')

		axes[1].plot(x, y1)
		axes[1].set_title('Probability Distribution of Data')
		axes[1].set_ylabel('Probability Density')

		axes[2].plot(x, y2)
		axes[2].set_title('Cumulative Distribution of Data')
		axes[2].set_ylabel('Probability')

		plt.show()

		return x, y1, y2
		
	def __add__(self, other):
		
		"""
		Function to add together two Gaussian distributions
		
		Args:
			other (Gaussian): Gaussian instance
			
		Returns:
			Gaussian: Gaussian distribution
			
		"""
		
		result = Gaussian()
		
		result.mean = self.mean + other.mean
		result.stdev = math.sqrt(self.stdev ** 2 + other.stdev ** 2)
		
		return result

	def __sub__(self, other):
		
		"""
		Function to subtract together two Gaussian distributions
		
		Args:
			other (Gaussian): Gaussian instance
			
		Returns:
			Gaussian: Gaussian distribution
			
		"""
		
		result = Gaussian()
		
		result.mean = self.mean - other.mean
		result.stdev = math.sqrt(self.stdev ** 2 + other.stdev ** 2)
		
		return result	
		
	def __repr__(self):
	
		"""
		Function to output the characteristics of the Gaussian instance
		
		Args:
			None
		
		Returns:
			string: characteristics of the Gaussian
		
		"""
		
		return "mean = {}, standard deviation = {}".format(self.mean, self.stdev)



import math
import matplotlib.pyplot as plt
import numpy as np
import random
from .GeneralDistribution import Distribution

class Binomial(Distribution):
    """ 
    Binomial distribution class for calculating and visualizing a Binomial distribution.
    
    Attributes:
        mean (float) representing the mean value of the distribution
        stdev (float) representing the standard deviation of the distribution
        data_list (list of floats) a list of floats to be extracted from the data file
        p (float) representing the probability of an event occurring
        n (int) number of trials            
    """
    
    
    def __init__(self, prob = random.uniform(0,1), size = 20):
                
        self.n = size
        self.p = prob
        Distribution.__init__(self, self.calculate_mean(), self.calculate_stdev())
        self.data = np.random.binomial(self.n, self.p, 500)
    
                        
    
    def calculate_mean(self):
    
        """
        Function to calculate the mean from p and n
        
        Args: 
            None
        
        Returns: 
            float: mean of the data set
    
        """
        
        self.mean = self.p * self.n
                
        return self.mean



    def calculate_stdev(self):

        """
        Function to calculate the standard deviation from p and n.
        
        Args: 
            None
        
        Returns: 
            float: standard deviation of the data set
    
        """
        
        self.stdev = math.sqrt(self.n * self.p * (1 - self.p))
        
        return self.stdev
        
        
    def read_data_file(self, file_name):
    
        """
        Function to read in data from a txt file. The txt file should have
        one number (float) per line. The numbers are stored in the data attribute. 
        After reading in the file, the mean and standard deviation are calculated
        
        Args: 
            file_name (string): name of a file to read from
        
        Returns: 
            float: the p value
            float: the n value
    
        """

        Distribution.read_data_file(self, file_name)

        try:
            npdata = np.array(self.data)
            assert np.array_equal(npdata, npdata.astype(bool)), 'data contains values other than 0 or 1'
        except AssertionError as error:
            raise

        self.n = len(self.data)
        self.p = 1.0 * sum(self.data) / self.n
        self.mean = self.calculate_mean()
        self.stdev = self.calculate_stdev()
        
        return self.p, self.n

    
        
    def plot_bar(self):

        """
        Function to output a histogram of the instance variable data using 
        matplotlib pyplot library.
        
        Args:
            None
            
        Returns:
            None
        """
                
        plt.bar(x = ['0', '1'], height = [(1 - self.p) * self.n, self.p * self.n])
        plt.title('Bar Chart of Data')
        plt.xlabel('Outcome')
        plt.ylabel('Count')
        plt.show()
        
        
    def pmf(self, k):

        """
        Probability mass function calculator for the binomial distribution.
        
        Args:
            k (unsigned int): point for calculating the probability mass function. k is in support (the set{0, 1, ..., n})
        
        Returns:
            float: probability density function output
        """
        
        try:
            assert k in range(self.n + 1), 'k isn\'t in support'
        except AssertionError as error:
            raise

        n_choose_k = math.factorial(self.n) / (math.factorial(k) * (math.factorial(self.n - k)))
        prob_of_k_successes = (self.p ** k)
        prob_rest_are_failures = (1 - self.p) ** (self.n - k)
        
        return n_choose_k * prob_of_k_successes * prob_rest_are_failures


    def cdf(self, k):

        """
        Cumulative distribution function calculator for the binomial distribution.
        
        Args:
            k (unsigned float): point for calculating the cumulative distribution function.
        
        Returns:
            float: cumulative density function output
        """
        
        try:
            assert k >= 0, 'k isn\'t nonnegative'
        except AssertionError as error:
            raise

        return sum([self.pmf(x) for x in range(math.floor(k) + 1)])        

    def plot_pmf_cdf(self):

        """
        Function to plot the pmf and cdf of the binomial distribution
        
        Args:
            None
        
        Returns:
            list: x values for the both plot
            list: y1 values for the pmf plot
            list: y2 values for cdf plot
            
        """
        
        x = []
        y1 = []
        y2 = []
        
        # calculate the x values to visualize
        for i in range(self.n + 1):
            x.append(i)
            y1.append(self.pmf(i))
            y2.append(self.cdf(i))

        # make the plots
        fig, axes = plt.subplots(2,sharex=True)
        fig.subplots_adjust(hspace=.5)
        
        axes[0].plot(x, y1)
        axes[0].set_title('Probability Distribution of Outcomes')
        axes[0].set_ylabel('Probability Density')

        axes[1].plot(x, y2)
        axes[1].set_title('Cumulative Distribution of Outcomes')
        axes[1].set_ylabel('Probability')

        plt.show()

        return x, y1, y2
        
    def __add__(self, other):
        
        """
        Function to add together two Binomial distributions with equal p
        
        Args:
            other (Binomial): Binomial instance
            
        Returns:
            Binomial: Binomial distribution
            
        """
        
        try:
            assert self.p == other.p, 'p values are not equal'
        except AssertionError as error:
            raise
        
        result = Binomial()
        result.n = self.n + other.n
        result.p = self.p
        result.calculate_mean()
        result.calculate_stdev()
        
        return result
        
        
    def __repr__(self):
    
        """
        Function to output the characteristics of the Binomial instance
        
        Args:
            None
        
        Returns:
            string: characteristics of the Gaussian
        
        """
        
        return "mean = {}, standard deviation = {}, p = {}, n = {}".\
        format(self.mean, self.stdev, self.p, self.n)


import matplotlib.pyplot as plt
import numpy as np
from .BinomialDistribution import Binomial

class Bernoulli(Binomial):
    """ 
    Bernoulli distribution class for calculating and visualizing a Bernoulli distribution.
    A Bernoulli distribution is a special case of a Binomial distribution (n = 1)
    
    Attributes:
        mean (float) representing the mean value of the distribution
        stdev (float) representing the standard deviation of the distribution
        data_list (list of floats) a list of floats to be extracted from the data file
        p (float) representing the probability of an event occurring
    """
    
    
    def __init__(self, prob = .5):

        self.n = 1        
        self.p = prob
        
        Binomial.__init__(self, self.p, self.n)

    def read_data_file(self):
        pass

    def pmf(self, k):

        """
        Probability mass function calculator for the bernoulli distribution.
        
        Args:
            k (unsigned int): point for calculating the probability mass function. k is in support (the set{0, 1})
        
        Returns:
            float: probability density function output
        """
        
        try:
            assert k in range(2), 'k isn\'t in support'
        except AssertionError as error:
            raise
        
        return 1 - self.p if k == 0 else self.p

    def cdf(self, k):

        """
        Cumulative distribution function calculator for the bernoulli distribution.
        
        Args:
            k (float): point for calculating the cumulative distribution function.
        
        Returns:
            float: cumulative density function output
        """
        if(k < 0): return 0
        return 1 - self.p if k >=0 and k < 1 else 1 

    def plot_pmf_cdf(self):

        """
        Function to plot the pmf and cdf of the Bernoulli distribution
        
        Args:
            None
        
        Returns:
            list: x values for the pmf plot
            list: y values for the pmf plot
            list: x2 values for the cdf plot
            list: y2 values for the cdf plot
        """
        x = [0,1]
        y = [self.pmf(0), self.pmf(1)]
        x2 = np.arange(-2, 2, .05).tolist()
        y2 = map(self.cdf, x2)

        # make the plots
        fig, axes = plt.subplots(2)
        fig.subplots_adjust(hspace=.5)
        axes[0].plot(x, y, 'o')
        axes[0].set_title('Probability Distribution of Outcomes')
        axes[0].set_ylabel('Probability Density')
        axes[0].set_ylim([0,1])
        axes[0].set_xticks([0, 1])
        axes[0].axvline(x[0],0, y[0])
        axes[0].axvline(x[1],0, y[1])

        axes[1].plot(x2, y2)
        axes[1].set_title('Cumulative Distribution of Outcomes')
        axes[1].set_ylabel('Probability')

        plt.show()

        return x, y, x2, y2



import math
import matplotlib.pyplot as plt
import numpy as np
from .GeneralDistribution import Distribution

class Poisson(Distribution):
	""" 
	Poisson distribution class for calculating and visualizing a Poisson distribution.
	
	Attributes:
		mean (float) representing the mean value of the distribution
		stdev (float) representing the standard deviation of the distribution
		data_list (list of floats) a list of floats to be extracted from the data file
		lam (float) mean number of times an event occurs in a time interval
	"""
	
	
	def __init__(self, lam = 5):
				
		self.lam = lam
		
		Distribution.__init__(self, self.calculate_mean(), self.calculate_stdev())
		self.data = np.random.poisson(self.lam, 500)
	
						
	
	def calculate_mean(self):
	
		"""
		Function to calculate the mean from lam

		Args: 
			None
		
		Returns: 
			float: mean of the data set
	
		"""
		
		self.mean = self.lam
				
		return self.mean



	def calculate_stdev(self):

		"""
		Function to calculate the standard deviation from lam
		
		Args: 
			None
		
		Returns: 
			float: standard deviation of the data set
	
		"""
		
		self.stdev = math.sqrt(self.lam)
		
		return self.stdev
 

	def read_data_file(self, file_name):
	
		"""
		Function to read in data from a txt file. The txt file should have
		one number (float) per line. The numbers are stored in the data attribute. 
		After reading in the file, the mean and standard deviation are calculated
		
		Args: 
			file_name (string): name of a file to read from
		
		Returns: 
			float: the lam value
	
		"""
		
		Distribution.read_data_file(self, file_name)

		self.lam = 1.0 * sum(self.data)/len(self.data)

		self.mean = self.calculate_mean()
		self.stdev = self.calculate_stdev()
		
		return self.lam

	def plot_histogram(self):
	
		"""
		Function to output a histogram of the instance variable data using 
		matplotlib pyplot library.
		
		Args:
			None
			
		Returns:
			None
		"""
		plt.hist(self.data)
		plt.title('Histogram of Data')
		plt.xlabel('Data')
		plt.ylabel('Count')
		plt.show()

	def pmf(self, k):

		"""
		Probability mass function calculation for the poisson distribution.
		
		Args:
			k (unsigned int): point for calculating the probability density function
		
		Returns:
			float: probability density function output
		"""

		try:
			assert (isinstance(k, int) or k.is_integer()) and k >= 0, 'k isn\'t a natural number'
		except AssertionError as error:
			raise

		return 1.0 * math.exp(-self.lam)*(self.lam ** k)/math.factorial(k)
		
	def cdf(self, k):

		"""
		Cumulative distribution function calculator for the poisson distribution.
        
        Args:
			k (unsigned float): point for calculating the cumulative distribution function.
        
 		Returns:
			float: cumulative density function output
		"""

		try:
			assert k >= 0, 'isn\'t nonnegative'
		except AssertionError as error:
			raise

		return sum([self.pmf(x) for x in range(int(math.floor(k)) + 1)])

	def plot_histogram_pmf_cdf(self):

		"""
		Function to plot the pmf of the poisson distribution
		
		Args:
			None
		
		Returns:
			list: x values for the both plot
			list: y1 values for the pmf plot
			list: y2 values for the cdf plot
		"""

		x = list(set(self.data))
		x.sort()
		y1 = []
		y2 = []
		
		# calculate the y values to visualize
		for elem in x:
			y1.append(self.pmf(elem))
			y2.append(self.cdf(elem))

		# make the plot
		fig, axes = plt.subplots(3,sharex=True)
		fig.subplots_adjust(hspace=.5)
		
		axes[0].hist(self.data, len(x), density=True)
		axes[0].set_title('Normed Histogram of Data')
		axes[0].set_ylabel('Density')

		axes[1].plot(x, y1)
		axes[1].set_title('Probability Distribution for Data')
		axes[1].set_ylabel('Probability Density')

		axes[2].plot(x, y2)
		axes[2].set_title('Cumulative Distribution for Data')
		axes[2].set_ylabel('Probability')

		plt.show()

		return x, y1, y2
		
	def __add__(self, other):
		
		"""
		Function to add together two Poisson distributions
		
		Args:
			other (Poisson): Poisson instance
			
		Returns:
			Poisson: Poisson distribution
			
		"""
		
		result = Poisson()
		result.lam = self.lam + other.lam
		result.calculate_mean()
		result.calculate_stdev()
		
		return result
		
		
	def __repr__(self):
	
		"""
		Function to output the characteristics of the Poisson instance
		
		Args:
			None
		
		Returns:
			string: characteristics of the Gaussian
		
		"""
		
		return "mean = {}, standard deviation = {}, lam = {}".\
		format(self.mean, self.stdev, self.lam)


from setuptools import setup

setup(
	name='prob_distrs',
    version='0.3',
    description='Some popular probability distributions',
    packages=['prob_distrs'],
    author="Aditya Vadlamani",
    author_email = 'aditya.t.vadlamani@gmail.com',
    url = 'https://github.com/AdityaVadlamani/prob-distrs',
    zip_safe=False
)
